﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.MultiObjectiveTestFunction;

namespace HeuristicLab.Problems.TestFunctions {
  [Item("DTLZ6", " http://repository.ias.ac.in/81671/ [30.11.15]")]
  [StorableClass]
  public class DTLZ6 : MultiObjectiveTestFunction {

    private int actualSolutionSize = 2;
    public override int ActualSolutionSize {
      get {
        return actualSolutionSize;
      }

      set {
        actualSolutionSize = value;
      }
    }

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { 0, 1 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[actualSolutionSize];
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {

        return int.MaxValue;
      }
    }

    public override int MinimumProblemSize {
      get {
        return Math.Max(2, ActualSolutionSize);
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    [StorableConstructor]
    protected DTLZ6(bool deserializing) : base(deserializing) { }
    protected DTLZ6(DTLZ6 original, Cloner cloner) : base(original, cloner) { }
    public DTLZ6() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DTLZ6(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      return Evaluate(r, ActualSolutionSize);
    }

    private double[] Evaluate(RealVector r, int objectives) {
      if (r.Length < objectives) {
        throw new Exception("The dimensionality of the problem(ProblemSize) must be larger than or equal to the dimensionality of the solution(SolutionSize) ");
      }
      double[] res = new double[objectives];

      //calculate g(Xm)
      double g = 0, length = 0;
      for (int i = objectives; i < r.Length; i++) {
        g += r[i];
        length += r[i] * r[i];
      }
      length = Math.Sqrt(length);
      g = 1.0 + 9.0 / length * g;
      if (length == 0) { g = 1; }

      //calculating f0...fM-2
      for (int i = 0; i < objectives - 1; i++) {
        res[i] = r[i];
      }
      //calculate fM-1
      double h = objectives;
      for (int i = 0; i < objectives - 1; i++) {
        h -= res[i] / (1 + g) * (1 + Math.Sin(3 * Math.PI * res[i]));
      }
      res[objectives - 1] = (1 + g) * h;

      return res;
    }
  }
}
