﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.MultiObjectiveTestFunction;

namespace HeuristicLab.Problems.TestFunctions {
  [Item("Kursawe", "from // http://darwin.di.uminho.pt/jecoli/index.php/Multiobjective_example [30.11.2015]")]
  [StorableClass]
  public class Kursawe : MultiObjectiveTestFunction {

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { -5, 5 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[] { false, false };
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {
        return 2;
      }
    }

    public override int MinimumProblemSize {
      get {
        return 3;
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    public override int ActualSolutionSize {
      get {
        return 2;
      }

      set {
      }
    }

    [StorableConstructor]
    protected Kursawe(bool deserializing) : base(deserializing) { }
    protected Kursawe(Kursawe original, Cloner cloner) : base(original, cloner) { }
    public Kursawe() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Kursawe(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      //objective 1
      double f0 = 0.0;
      for (int i = 0; i < 2; i++) {
        f0 += -10 * Math.Exp(-0.2 * Math.Sqrt(r[i] * r[i] + r[i + 1] * r[i + 1]));
      }
      //objective2
      double f1 = 0.0;
      for (int i = 0; i < 3; i++) {
        f1 += Math.Pow(Math.Abs(r[i]), 0.8) + 5 * Math.Sin(Math.Pow(r[i], 3));
      }

      return new double[] { f0, f1 };
    }
  }
}
